/***************************************************************************************
 *
 *  IMPORTANT: READ BEFORE DOWNLOADING, COPYING, INSTALLING OR USING.
 *
 *  By downloading, copying, installing or using the software you agree to this license.
 *  If you do not agree to this license, do not download, install, 
 *  copy or use the software.
 *
 *  Copyright (C) 2014-2025, Happytimesoft Corporation, all rights reserved.
 *
 *  Redistribution and use in binary forms, with or without modification, are permitted.
 *
 *  Unless required by applicable law or agreed to in writing, software distributed 
 *  under the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
 *  CONDITIONS OF ANY KIND, either express or implied. See the License for the specific
 *  language governing permissions and limitations under the License.
 *
****************************************************************************************/

#ifndef _HTTP_H_
#define _HTTP_H_

#include "sys_buf.h"
#include "ppstack.h"


/***************************************************************************************/

typedef enum http_request_msg_type
{
    HTTP_MT_NULL = 0,
    HTTP_MT_GET,
    HTTP_MT_HEAD,
    HTTP_MT_MPOST,
    HTTP_MT_MSEARCH,
    HTTP_MT_NOTIFY,
    HTTP_MT_POST,
    HTTP_MT_SUBSCRIBE,
    HTTP_MT_UNSUBSCRIBE,
} HTTP_MT;

typedef enum http_content_type
{
    CTT_NULL = 0,
    CTT_SDP,
    CTT_TXT,
    CTT_HTM,
    CTT_XML,
    CTT_BIN,
    CTT_JPG,
    CTT_RTSP_TUNNELLED,
    CTT_MULTIPART,
    CTT_FLV,
    CTT_JSON
} HTTPCTT;

/***************************************************************************************/

#define ctt_is_string(type) (type == CTT_XML || type == CTT_HTM || type == CTT_JSON || \
                             type == CTT_TXT || type == CTT_SDP)


typedef struct _http_msg_content
{
    uint32          msg_type;
    uint32          msg_sub_type;
    HDRV            first_line;

    PPSN_CTX        hdr_ctx;
    PPSN_CTX        ctt_ctx;

    int             hdr_len;
    int             ctt_len;
    HTTPCTT         ctt_type;
    char            boundary[256];
    int             keep_alive;

    char          * msg_buf;
    int             buf_offset;
} HTTPMSG;

/***************************************************************************************/

typedef struct http_client
{
    uint32          pass_through : 1;   // pass through received data flag
    uint32          https        : 1;   // https flag
    uint32          keep_alive   : 1;   // keep alive flag
    uint32          resv         : 29;
    
    SOCKET          cfd;                // client socket
    HT_SOCKADDR     sockaddr;           // remote socket address

    char            rcv_buf[2052];      // static receiving buffer
    char          * dyn_recv_buf;       // dynamic receiving buffer
    int             rcv_dlen;           // received data length
    int             hdr_len;            // http header length
    int             ctt_len;            // context  length
    HTTPCTT         ctt_type;           // context type
    char          * rbuf;               // pointer to rcv_buf or dyn_recv_buf
    int             mlen;               // sizeof(rcv_buf) or size of dyn_recv_buf

    void          * http_srv;           // pointer to HTTPSRV
    int             use_count;          // use count
    void          * use_count_mutex;    // use count mutex
    uint32          last_time;          // last time

    void          * userdata;           // user data
    void          * userdata_mutex;     // user data mutex
    uint32          protocol;           // protocol, rtsp over http, websocket etc

    void          * ssl;                // https SSL
    void          * ssl_mutex;          // https SSL mutex
} HTTPCLN;

typedef struct http_req
{
    uint32          need_auth : 1;      // need auth flag
    uint32          https     : 1;      // https flag
    uint32          resv      : 30;
    
    SOCKET          cfd;                // client socket    
    uint32          port;               // server port
    char            host[256];          // server host
    char            url[256];           // the request url
    uint16          family;             // address family

    char            action[256];        // action
    char            rcv_buf[2052];      // static receiving buffer
    char          * dyn_recv_buf;       // dynamic receiving buffer
    int             rcv_dlen;           // received data length
    int             hdr_len;            // http header length            
    int             ctt_len;            // context  length
    char            boundary[256];      // boundary, for CTT_MULTIPART
    char          * rbuf;               // pointer to rcv_buf or dyn_recv_buf
    int             mlen;               // sizeof(rcv_buf) or size of dyn_recv_buf

    HTTPMSG       * rx_msg;             // rx message

    int             auth_mode;          // 0 - baisc; 1 - digest
    HD_AUTH_INFO    auth_info;          // http auth information

    void          * ssl;                // https SSL 
    void          * ssl_mutex;          // https SSL mutex
} HTTPREQ;

/***************************************************************************************/

/**
 * @brief http message callback 
 * 
 * If rx_msg is NULL, the callback should call the http_free_used_cln function to delete p_cln
 *
 * @param p_cln http client pointer
 * @param rx_msg http message
 * @param userdata user data
 *
 * @return If the callback is responsible for deleting rx_msg, 
 *         it returns TRUE, otherwise it returns FALSE
 */
typedef BOOL (*http_msg_cb)(HTTPCLN * p_cln, HTTPMSG * rx_msg, void * userdata);

/**
 * @brief HTTP raw data receiving callback
 *  only called when p_cln->pass_through is set to 1
 *  Used for RTSP over HTTP and RTSP over websocket        
 *
 * @param p_cln http client pointer
 * @param buff data buffer
 * @param buflen data buffer length
 *
 */
typedef void (*http_data_cb)(HTTPCLN * p_cln, char * buff, int buflen, void * userdata);

/**
 * @brief http new connection callback 
 *
 * @param p_srv http server pointer
 * @param addr http client connection address
 *
 * @return TRUE, accept the new connection, FALSE, reject the new connection 
 */
typedef BOOL (*http_conn_cb)(void * p_srv, struct sockaddr * addr, void * userdata);

typedef struct http_srv_s
{
    uint32          r_flag  : 1;        // data receiving flag
    uint32          https   : 1;        // https flag
    uint32          ipv6    : 1;        // ipv6 flag
    uint32          resv    : 29;
    
    HT_SOCKADDR     sockaddr;           // server socket address
    SOCKET          ipv4_fd;            // ipv4 socket
    SOCKET          ipv6_fd;            // ipv6 socket
    char            ipv4_host[64];      // ipv4 local server address
    char            ipv6_host[128];     // ipv6 local server address
    uint32          max_cln_nums;       // max client number

    PPSN_CTX      * cln_fl;             // client free list
    PPSN_CTX      * cln_ul;             // client used list

    uint32          rx_num;             // data receiving thread numbers
    pthread_t     * rx_tid;             // data receiving thread id

    void          * mutex_cb;           // cabllback mutex
    http_msg_cb     msg_cb;             // http message callback
    http_data_cb    data_cb;            // http data callback
    http_conn_cb    conn_cb;            // http connection callback
    void          * msg_user;           // http message callback user data
    void          * data_user;          // http data callback user data
    void          * conn_user;          // http connection callback user data

    int             ep_fd;              // epoll fd
    void          * ep_events;          // epoll events
    int             ep_event_num;       // epoll event number    

    char            cert_file[256];     // cert file name
    char            key_file[256];      // key file name
    void          * ssl_ctx;            // ssl context
} HTTPSRV;

#endif




